# -*- coding: utf-8; -*-
# Authors::   Kiyoshi Ikehara <kiyoshi.ikehara@gree.net>
# Copyright:: Copyright (C) GREE, Inc. 2011.
# License::   MIT-style

require 'flare/tools/index_server'
require 'flare/util/conversion'
require 'flare/util/logging'
require 'flare/tools/cli/sub_command'

module Flare
  module Tools

    module Cli
      class Ping < SubCommand
        include Flare::Util::Conversion
        include Flare::Util::Logging

        myname :ping
        desc   "ping"
        usage  "ping [hostname:port] ..."

        def setup
          super
          @optp.on('--wait',            "wait for OK responses from nodes") {@wait = true}
        end

        def initialize
          super
          @wait = false
        end

        def execute(config, args)

          hosts = args.map do |arg|
            hostname, port, rest = arg.split(':', 3)
            if !rest.nil? || hostname.nil? || hostname.empty? || port.nil? || port.empty?
              error "invalid argument '#{arg}'. it must be hostname:port."
              return S_NG
            end
            begin
              ipaddr = Resolv.getaddress(hostname)
            rescue Resolv::ResolvError
              error "unknown host '#{hostname}'"
              return S_NG
            end
            [hostname, port]
          end

          hosts.each do |hostname, port|
            resp = nil
            until resp
              begin
                debug "trying..."
                interruptible do
                  Flare::Tools::Stats.open(hostname, port, @timeout) do |s|
                    resp = s.ping
                  end
                end
              rescue IOError
                return S_NG
              rescue
                unless @wait
                  puts "#{hostname}:#{port} is down"
                  return S_NG
                end
                interruptible {sleep 1}
              end
            end
          end

          puts "alive"
          S_OK
        end

      end
    end
  end
end
